
// Tom Hall
// (c) 2014-2015

SndRecGUI {

  var <chans, chanMenu, <recDir, fileField, <filePath;
  var bitMenu, <bitDepth, <win, <fileName, recBtn;
  var flow, fileLocs, locNames, recReady, <recHeader;
  var <headerMenu, server, customLoc, <isRecording=false;

  *new {|server, customLoc |
    ^super.new.init(server, customLoc);
  }

  init { |aserver, acustomLoc|
    server = aserver;
    customLoc = acustomLoc;

    if(server.isNil, {
      "Server argument is nil. Please pass in a server".error;
      ^this
    });
    recHeader = "aiff";
    recReady = false;
    chans=2;
    bitDepth="int16";
    locNames = ["Desktop", "default"];
    recDir = "~/Desktop/".standardizePath;

    fileLocs = Array.with(
      recDir,
      thisProcess.platform.recordingsDir ++ "/";
    );

    if(customLoc.notNil, {
      customLoc = customLoc.withTrailingSlash;
      fileLocs = fileLocs.add(customLoc);
      locNames = locNames.add(customLoc);
    });

    this.makeGUI;

    ^this
  }

  makeGUI {
    var screenWidth, screenBounds, screenHeight;
    screenBounds = Window.screenBounds;
    screenWidth = screenBounds.width;
    screenHeight = screenBounds.height;
    win = Window.new("SndRecGUI", Rect(screenWidth-175, screenHeight-45, 170, 255), resizable: false);

    this.alwaysOnTop_(true);
    win.front;

    flow = win.addFlowLayout( 10@10, 10@5 );

    StaticText(win, Rect(10, 10, 75, 20)).string_("Num Chans");

    chanMenu = PopUpMenu.new(win, 65@20);
    chanMenu.items = ["2","1","4","3","5","6","7","8", "9", "10", "11", "12", "13", "14", "15", "16", "24", "36", "48"];
    chanMenu.action = { arg menu;
      chans =[2, 1, 4, 5, 6, 8][menu.value].postln;
    };

    flow.nextLine;

    StaticText(win, Rect(10, 10, 75, 20)).string_("Bit Depth");

    bitMenu = PopUpMenu.new(win, 65@20);
    bitMenu.items = ["int16","int24","float"];
    bitMenu.action = { arg menu;
      bitDepth = menu.item.postln;
    };

    StaticText(win, Rect(10, 10, 75, 20)).string_("Rec Header");

    headerMenu = PopUpMenu.new(win, 65@20);
    headerMenu.items = ["aiff","wav"];
    headerMenu.action = { arg menu;
      recHeader = menu.item.postln;
    };

    StaticText(win, Rect(10, 10, 120, 20)).string_("File location");

    flow.nextLine;

    bitMenu = PopUpMenu.new(win, 150@20);
    bitMenu.items = locNames;
    bitMenu.action = { arg menu;
      recDir = fileLocs[menu.value].postln;
    };

    flow.nextLine;

    StaticText(win, Rect(10, 10, 150, 20)).string_("File name");

    flow.nextLine;

    fileField = TextField(win, Rect(10, 10, 150, 25));
    fileField.string = "";
    fileField.background_(Color.white);

    fileField.keyDownAction_({ |v, char, mod, unicode, keycode|
      if(unicode == 13, {
        if(fileName.isNil,  {
          this.fileName_(v.string);
          }, {
            "filename already set".warn;
            fileField.string = fileName; // restore to what was already there
        });
      });
    });

    flow.nextLine;
    flow.nextLine; // spacer

    recBtn = Button(win, Rect(10, 10, 150, 50));
    recBtn.states_([
      ["Input file name\nthen press RETURN", Color.white, Color.black],
      ["RECORD", Color.black, Color.green],
      ["stop recording", Color.black, Color.red]
    ]);
    recBtn.action_({ arg butt;
      case {butt.value==0}{
        if(recReady, { // recording is finished or ready to begin
          server.stopRecording;
          fileField.background_(Color.white);
          fileField.string = "";
          recReady = false;
          isRecording = false;
          fileName = nil;
        });

      }
      {butt.value==1}{
        if(recReady, {
          this.prepareForRecord;
          }, {
            butt.value = 0; // no progress if file name not done
            "No input on filename".error;
        });
      }
      {butt.value==2}{
        server.record;
        isRecording = true;
      };
    });
    ^this;
  }


  alwaysOnTop_ {|bool|
    bool = bool ? true;
    win.alwaysOnTop_(bool);
    ^this;
  }

  record {
    if(recReady, {
      recBtn.valueAction = 2;
      }, {
        "Please first enter a filename".error;
    });
    ^this;
  }

  stopRecording {
    recBtn.valueAction = 0;
    ^this;
  }

  stop {
    this.stopRecording;
    ^this;
  }

  fileName_ {|string|
    if(isRecording.not and: {fileName.isNil}, { // return
      string = string ? "";
      string = this.checkFileName(string);
      fileField.string = string;
      fileField.background_(Color.green);
      recReady = true;
      recBtn.valueAction = 1; // move rec button along
      },
      {
        "Current recording not finished".warn;
    });
  }

  checkFileName {|string|
    if(string.isEmpty, {
      // from Server source:
      // temporary kludge to fix Date's brokenness on windows
      if(thisProcess.platform.name == \windows, {
        string = "SC_" ++ Main.elapsedTime.round(0.01) ++ "." ++ recHeader;
        }, {
          string = "SC_" ++ Date.localtime.stamp ++ "." ++ recHeader;
      });
    });
    if(string.containsi("." ++ recHeader.copyRange(0, 2)).not, {
      string = string++"." ++ recHeader.copyRange(0, 2);
    });
    ^fileName = string;
  }

  // after first button press
  prepareForRecord  {
    server.waitForBoot({
      server.recChannels = chans;
      server.recSampleFormat = bitDepth;
      server.recHeaderFormat = recHeader;
      filePath = recDir++fileName;
      filePath.postln;
      server.prepareForRecord(filePath);
    });
    ^this;
  }

  close { win.close;^this }

}


